<?php
/**
 * @file Contains Drupal\force_password_change\Controller\ForcePasswordChangeController
 *
 * Handles page requests for pages in the Force Password Change module
 */

namespace Drupal\force_password_change\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountProxy;
use Drupal\Core\Url;
use Drupal\user\Entity\Role;
use Symfony\Component\DependencyInjection\ContainerInterface;

class ForcePasswordChangeController extends ControllerBase
{
	/**
	 * A form builder object
	 *
	 * @var \Drupal\Core\Form\FormBuilderInterface
	 */
	protected $formBuilder;

	/**
	 * The date formatter service
	 *
	 * @var Drupal\Core\Datetime\DateFormatter
	 */
	protected $dateFormatter;

	/**
	 * The current user
	 *
	 * @var Drupal\Core\Session\AccountProxy
	 */
	protected $currentUser;

	/**
	 * The database connection
	 *
	 * @var Drupal\Core\Database\Connection
	 */
	protected $connection;

	/**
	 * {@inheritdoc}
	 */
	public static function create(ContainerInterface $container)
	{
		return new static
		(
			$container->get('form_builder'),
			$container->get('date.formatter'),
			$container->get('current_user'),
			$container->get('database')
		);
	}

	/**
	 * Constructs a FriendController object.
	 *
	 * @param \Drupal\Core\Form\FormBuilderInterface $formBuilder
	 *   The form builder service.
	 * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
	 *   The date formatter service
	 * @param \Drupal\Core\Session\AccountProxy $currentUser
	 *   The current user
	 * @param \Drupal\Core\Database\Connection $connection
	 *   The database connection
	 */
	public function __construct(FormBuilderInterface $formBuilder, DateFormatter $dateFormatter, AccountProxy $currentUser, Connection $connection)
	{
		$this->formBuilder = $formBuilder;
		$this->dateFormatter = $dateFormatter;
		$this->currentUser = $currentUser;
		$this->connection = $connection;
	}

	public function adminPage()
	{
		$page = array
		(
			'#prefix' => '<div id="force_password_change_admin_page">',
			'#suffix' => '</div>',
			'form' => $this->formBuilder->getForm('Drupal\force_password_change\Form\AdminForm'),
		);

		return $page;
	}

	public function roleListPage($rid)
	{
		$page = array
		(
			'#prefix' => '<div id="force_password_change_role_page">',
			'#suffix' => '</div>',
		);

		$role = Role::load($rid);
		if($role && $role->id() != 'anonymous')
		{
			// Get the time of the last forced change for the role
			$last_change = $this->connection->query('SELECT last_force FROM {force_password_change_roles} WHERe rid = :rid', array(':rid' => $rid))->fetchCol();

			// Format the date into a usable string
			$last_change = ($last_change[0] != '') ? $this->dateFormatter->format($last_change[0], 'short') : t('Never');
			$variables['last_change'] = $last_change;
	
			// Get a list of users that have a pending forced password change
			$pending_query = $this->connection->select('users_data', 'ud')->extend('Drupal\Core\Database\Query\PagerSelectExtender');
			$alias = $pending_query->join('force_password_change_users', 'fpcu', 'fpcu.uid = ud.uid');
			$pending_query
				->addTag('force_password_change_pending_users')
				->fields('ud', array('uid'))
				->fields($alias, array('last_force', 'last_password_change'))
				->limit(100)
				->condition('ud.module', 'force_password_change')
				->condition('ud.name', 'force_password_change')
				->condition('ud.value', 1);
			// If the role is anything other than the authenticated users role, we need to
			// limit the users to the members of that role
			if($rid != 'authenticated')
			{
				$alias2 = $pending_query->join('user__roles', 'ur', 'ur.entity_id = ud.uid');
				$pending_query->condition($alias2 . '.roles_target_id', $rid);
			}
			$pending_user_ids = $pending_query->execute()->fetchCol();

			$pending_users = user_load_multiple($pending_user_ids);

			// build the header for the table
			$header = array(t('Username'), t('Last Forced Password Change'), t('Last Password Change'));

			// Next build the rows of the table, and the stats that will be included for each user shown
			$rows = array();
			$force_password_change_installation_date = \Drupal::config('force_password_change.installation_date')->get('unix_date');
			$forced_uids = \Drupal::config('force_password_change.first_time_uids')->get('uids');
			foreach($pending_users as $pending_user)
			{
				$row = array();

				if($this->currentUser->hasPermission('access user profiles'))
				{
					$url = Url::fromRoute('entity.user.canonical', array('user' => $pending_user->id()));
					$row[] = Link::fromTextAndUrl($pending_user->getDisplayName(), $url);
				}
				else
				{
					$row[] = $pending_user->getDisplayName();
				}

				if($pending_user->last_force != '')
				{
					$last_force = $this->dateFormatter($pending_user->last_force, 'short');
				}
				elseif(\Drupal::config('force_password_change.first_time_login_password_change')->get('change') && $pending_user->getCreatedTime() > $force_password_change_installation_date)
				{
					$last_force = t('First login');
				}
				else
				{
					if((count($forced_uids) && isset($forced_uids[$pending_user->id()])) || $pending_user->last_password_change != '')
					{
						$last_force = t('First login');
					}
					else
					{
						$last_force = t('Never');
					}
				}

				$row[] = $last_force;
				$row[] = ($pending_user->last_password_change != '') ? $this->dateFormatter($pending_user->last_password_change, 'short') : t('Never');

				$rows[] = $row;
			}

			// Build the table containing the retreived data
			$page['pending_users_table'] = array
			(
				'header' => array
				(
					'#prefix' => '<h2>',
					'#suffix' => '</h2>',
					'#markup' => t('Users in this role with pending password changes'),
				),
				'table' => array
				(
					'#type' => 'table',
					'#header' => $header,
					'#rows' => $rows,
					'#empty' => t('No users found'),
				),
				'pager' => array
				(
					'#type' => 'pager',
					'#quantity' => 5,
				),
			);

			// Perform the same steps as the previous table, for users who do not have a pending forced password change
			$nonpending_query = $this->connection->select('users_data', 'ud')->extend('Drupal\Core\Database\Query\PagerSelectExtender');
			$alias = $nonpending_query->join('force_password_change_users', 'fpcu', 'fpcu.uid = ud.uid');

			$nonpending_query
				->addTag('force_password_change_nonpending_users')
				->fields('ud', array('uid'))
				->fields($alias, array('last_force', 'last_password_change'))
				->limit(100)
				->condition('ud.module', 'force_password_change')
				->condition('ud.name', 'force_password_change')
				->condition('ud.value', 0);

			if($rid != 'authenticated')
			{
				$alias2 = $nonpending_query->join('user__roles', 'ur', 'ur.entity_id = ud.uid');
				$nonpending_query->condition($alias2 . '.roles_target_id', $rid);
			}

			$nonpending_uids = $nonpending_query->execute()->fetchCol();
			$nonpending_users = user_load_multiple($nonpending_uids);

			$header = array(('Username'), t('Last Forced Password Change'), t('Last Password Change'));
			$rows = array();
			foreach($nonpending_users as $nonpending_user)
			{
				$row = array();
				if($this->currentUser->hasPermission('access user profiles'))
				{
					$url = Url::fromRoute('entity.user.canonical', array('user' => $nonpending_user->id()));
					$row[] = Link::fromTextAndUrl($nonpending_user->getDisplayName(), $url);
				}
				else
				{
					$row[] = $pending_user->getDisplayName();
				}

				if($nonpending_user->last_password_change != '')
				{
					$last_force = $this->dateFormatter($nonpending_user->last_password_change, 'short');
				}
				elseif(\Drupal::config('force_password_change.first_time_login_password_change')->get('change') && $nonpending_user->getCreatedDate() > $force_password_change_installation_date)
				{
					$last_force = t('First login');
				}
				else
				{
					if((count($forced_uids) && isset($forced_uids[$nonpending_user->id()])) || $nonpending_user->last_password_change != '')
					{
						$last_force = t('First login');
					}
					else
					{
						$last_force = t('Never');
					}
				}
				$row[] = $last_force;
				$row[] = $nonpending_user->last_password_change != '' ? $this->dateFormatter($nonpending_user->last_password_change, 'short') : t('Never');
				$rows[] = $row;
			}

			// Build the table containing the retrieved data
			$page['nonpending_users_table'] = array
			(
				'header' => array
				(
					'#prefix' => '<h2>',
					'#suffix' => '</h2>',
					'#markup' => t('Users in this role without pending password changes'),
				),
				'table' => array
				(
					'#type' => 'table',
					'#header' => $header,
					'#rows' => $rows,
					'#empty' => t('No users found'),
				),
				'pager' => array
				(
					'#type' => 'pager',
					'#quantity' => 5,
				),
			);

			$page['form'] = $this->formBuilder()->getForm('Drupal\force_password_change\Form\RoleForceForm', $role);
		}
		else
		{
			$page['invalid_role'] = array
			(
				'#prefix' => '<p>',
				'#suffix' => '</p>',
				'#markup' => t('Invalid role'),
			);
		}

		return $page;
	}
}
