<?php

/**
 * @file Contains Drupal\force_password_change\Form\AdminForm
 *
 * Form handler for the form to add users as a friend
 */

namespace Drupal\force_password_change\Form;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\force_password_change\Service\ForcePasswordChangeService;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class AdminForm extends FormBase
{
	/**
	 * The config factory object
	 *
	 * @var Drupal\Core\Config\ConfigFactory
	 */
	protected $configFactory;

	/**
	 * The database connect object
	 *
	 * @var Drupal\Core\Database\Connection
	 */
	protected $connection;

	/**
	 * The force password change service
	 *
	 * @var Drupal\force_password_change\Service\ForcePasswordChangeService
	 */
	protected $passwordChangeService;

	/**
	 * Constructs an AdminForm object.
	 *
	 * @param \Drupal\Core\Config\ConfigFactory $configFactory
	 *   The config factory service.
	 * @param \Drupal\Core\Database\Connection $connection
	 *   The database connection object.
	 */
	public function __construct(ConfigFactory $configFactory, Connection $connection, ForcePasswordChangeService $passwordChangeService)
	{
		$this->configFactory = $configFactory;
		$this->connection = $connection;
		$this->passwordChangeService = $passwordChangeService;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function create(ContainerInterface $container)
	{
		return new static (
			$container->get('config.factory'),
			$container->get('database'),
			$container->get('force_password_change.service')
		);
	}

	public function getFormId()
	{
		return 'force_password_change_admin_form';
	}

	public function buildForm(array $form, FormStateInterface $form_state)
	{
		$form['#attached']['library'] = array
		(
			'force_password_change/settings_page',
		);

		$form['first_time_login_password_change'] = array
		(
			'#type' => 'checkbox',
			'#title' => t('Force password change on first-time login'),
			'#default_value' => $this->configFactory->get('force_password_change.first_time_login_password_change')->get('change'),
		);

		$form['login_only'] = array
		(
			'#type' => 'radios',
			'#title' => t('Check for pending password change'),
			'#options' => array(0 => t('on every page load'), 1 => t('on login only')),
			'#default_value' => (int) $this->configFactory->get('force_password_change.when_to_check')->get('login_only'),
			'#description' => t('Select when to check if a user has a pending password change. Checking on every page load adds a little overhead to every page load, but is the most secure method. Checking on login will only check if a change is pending when the user first logs in, but on sites where users may stay logged in for lengthy periods of time, it may be a long time before they are forced to change their password.'),
		);

		$roles = array();
		// Get stats for each of the roles on the site
		$all_roles = Role::loadMultiple();
		unset($all_roles[RoleInterface::ANONYMOUS_ID]);
		foreach($all_roles as $rid => $role)
		{
			if($rid == 'authenticated')
			{
				$user_count = db_query('SELECT COUNT(uid) FROM {users} WHERE uid > 0')->fetchField();
				$pending_count = db_query('SELECT COUNT(uid) FROM {users_data} WHERE module = :force_password_change AND name = :force_password_change AND value = 1', array(':force_password_change' => 'force_password_change'))->fetchField();
			}
			elseif($rid == 2)
			{
				$user_count = $this->connection->query('SELECT COUNT(uid) FROM {users_roles} WHERE roles_target_id = :rid', array(':rid' => $rid))->fetchField();
				$pending_count = db_query
				(
					'SELECT COUNT(u.uid) ' .
					'FROM {users_data} AS ud ' .
					'JOIN {users_roles} AS ur ' .
					'ON ur.uid = ud.uid ' .
					'WHERE ud.module = :force_password_change ' .
					'AND ud.name = :force_password_change ' .
					'AND ud.value = 1 ' .
					'AND ur.roles_target_id = :rid',
					array
					(
						':force_password_change' => 'force_password_change',
						':rid' => $rid,
					)
				)->fetchField();
			}

			$url = Url::fromRoute('force_password_change.admin.role.list', array('rid' => $rid));
			$link = Link::fromTextAndUrl(t('Details'), $url);
			$roles[$rid] = t(
				'@role (Users in role: @user_count | Users with pending forced password change: @pending_user_count | @details_link)',
				array
				(
					'@role' => $role->label(),
					'@user_count' => $user_count,
					'@pending_user_count' => $pending_count,
					'@details_link' => new FormattableMarkup('@link', array('@link' => $link->toString())),
				)
			);
		}

		$form['roles'] = array
		(
			'#type' => 'checkboxes',
			'#options' => $roles,
			'#title' => t('Force users in the following roles to change their password'),
			'#description' => t('Users will be forced to change their password either on their next page load, or on their next login, depending on the setting in "Check for pending password change". If pending password changes are checked on every page load, logged in users will be forced to immediately change their password, and after changing it will be redirected back to the page they were attempting to access.') . '<br />' . t('Note: When you return to this page, no roles will be selected. This is because this setting is a trigger, not a persistant state.'),
		);

		$expiry_data = $this->connection->query('SELECT rid, expiry, weight from {force_password_change_expiry} ORDER BY weight, rid');
		$expiry = array();
		foreach($expiry_data  as $data)
		{
			$expiry[$data->rid] = array
			(
				'expiry' => $data->expiry,
				'weight' => $data->weight,
			);
		}

		$form['expiry_data'] = array
		(
			'#type' => 'value',
			'#value' => $expiry,
		);

		$form['expiry'] = array
		(
			'#type' => 'fieldset',
			'#title' => t('Password Expiry'),
			'#collapsible' => TRUE,
		);

		$form['expiry']['expire_password'] = array
		(
			'#type' => 'checkbox',
			'#title' => t('Enable password expiration'),
			'#default_value' => $this->configFactory->get('force_password_change.expire_password')->get('expire'),
			'#description' => t('When this box is checked, passwords will be set to expire according to the rules set out below. If this box is un-checked, password expiry will be disabled, and the password expiry options below will be ignored.'),
		);

		$form['expiry']['header'] = array
		(
			'#markup' => '<p>' . t('Select the amount of time after which you would like users in a role to be automatically forced to change their password. Any users who do not change their password in this amount of time will be forced to change their password on their next login or page load (depending on the setting in "Check for pending password change"). If you do not wish passwords to expire for a certain role, leave/set the value for that role to zero.') . '</p>',
		);

		$form['expiry']['table'] = array
		(
			'#tree' => TRUE,
			'#type' => 'table',
			'#header' => array(t('Role'), t('Expire password after:'), t('Weight')),
			'#id' => 'force_password_change_role_expiry_table',
			'#tabledrag' => array
			(
				array
				(
					'action' => 'order',
					'relationship' => 'sibling',
					'group' => 'role-expiry-order-weight',
				),
			),
		);

		$time_periods = array
		(
			'hour' => 60 * 60,
			'day' => 60 * 60 * 24,
			'week' => 60 * 60 * 24 * 7,
			'year' => 60 * 60 * 24 * 365,
		);

		$time_options = array
		(
			'hour' => t('hours'),
			'day' => t('days'),
			'week' => t('weeks'),
			'year' => t('years'),
		);

		$heaviest_weight = 0;
		if(count($expiry))
		{
			foreach($expiry as $rid => $data)
			{
				$form['expiry']['table'][$rid] = array
				(
					'#weight' => $data['weight'],
					'#attributes' => array('class' => array('draggable')),
				);

				$form['expiry']['table'][$rid]['role'] = array
				(
					'#plain_text' => $all_roles[$rid]->label(),
				);

				$time_period_default = 0;
				$time_quantity_default = 0;
				if($data['expiry'] != '' && $data['expiry'])
				{
					$expires = $data['expiry'];
					if($expires % $time_periods['year'] === 0)
					{
						$time_period_default = 'year';
						$time_quantity_default = $expires / $time_periods['year'];
					}
					else
					{
						if($expires % $time_periods['week'] === 0)
						{
							$time_period_default = 'week';
							$time_quantity_default = $expires / $time_periods['week'];
						}
						else
						{
							if($expires % $time_periods['day'] === 0)
							{
								$time_period_default = 'day';
								$time_quantity_default = $expires / $time_periods['day'];
							}
							else
							{
								$time_period_default = 'hour';
								if($expires % $time_periods['hour'] === 0)
								{
									$time_quantity_default = $expires / $time_periods['hour'];
								}
							}
						}
					}
				}

				$form['expiry']['table'][$rid]['time']['time_quantity'] = array
				(
					'#type' => 'textfield',
					'#default_value' => $time_quantity_default,
					'#size' => 4,
				);

				$form['expiry']['table'][$rid]['time']['time_period'] = array
				(
					'#type' => 'select',
					'#options' => $time_options,
					'#default_value' => $time_period_default,
				);

				$form['expiry']['table'][$rid]['weight'] = array
				(
					'#type' => 'weight',
					'#title' => t('Weight for expiry roles'),
					'#title_display' => 'invisible',
					'#default_value' => ($data['weight'] != '') ? $data['weight'] : 0,
					'#attributes' => array('class' => array('role-expiry-order-weight')),
				);

				$heaviest_weight = ($data['weight'] != '') ? $data['weight'] : 0;
			}
		}

		foreach($roles as $rid => $r)
		{
			if(!isset($form['expiry']['table'][$rid]))
			{
				$heaviest_weight++;
				$form['expiry']['table'][$rid] = array
				(
					'#weight' => $heaviest_weight,
					'#attributes' => array('class' => array('draggable')),
				);

				$form['expiry']['table'][$rid]['role'] = array
				(
					'#markup' => $r,
				);

				$form['expiry']['table'][$rid]['time']['time_quantity'] = array
				(
					'#type' => 'textfield',
					'#default_value' => 0,
					'#size' => 4,
				);

				$form['expiry']['table'][$rid]['time']['time_period'] = array
				(
					'#type' => 'select',
					'#options' => $time_options,
				);

				$form['expiry']['table'][$rid]['weight'] = array
				(
					'#type' => 'weight',
					'#title' => t('Weight for expiry roles'),
					'#title_display' => 'invisible',
					'#default_value' => $heaviest_weight,
					'#attributes' => array('class' => array('role-expiry-order-weight')),
				);
			}
		}

		$form['expiry']['footer'] = array
		(
			'#markup' => '<p>' . t('Drag and drop the rows to set the priority for password expiry. The roles with the highest priority should be placed at the top of the list. If a user is a member of more than one role, then the time after which their password expires will be determined by whichever of their roles has the highest priority (highest in the list). Expiry rules for any roles of lower priority (lower in the list) will be ignored. As such, any roles lower in priority (below) the authenticated user role will effectively be ignored, since all users are members of the authenticated users role.') . '</p>',
		);

		$form['submit'] = array
		(
			'#type' => 'submit',
			'#value' => t('Submit'),
		);

		return $form;
	}

	public function submitForm(array &$form, FormStateInterface $form_state)
	{
		// First set some variable defaults
		$this->configFactory->getEditable('force_password_change.first_time_login_password_change')
			->set('change', (bool) $form_state->getValue('first_time_login_password_change'))
			->save();

		$this->configFactory->getEditable('force_password_change.when_to_check')
			->set('login_only', (bool) $form_state->getValue('login_only'))
			->save();

		$selected_roles = array();
		$au = FALSE;
		foreach($form_state->getValue('roles') as $rid)
		{
			// The authenticated user role. All users on the site will have a password change forced
			if($rid === 'authenticated')
			{
				// Update all user's {user} table
				$this->passwordChangeService->forceUsersPasswordChange();
				$selected_roles[] = $rid;
				// Since all users on the site have had their password change forced,
				// no other queries need to be run on the users table. However, we do want
				// to log the rid for each role. In order to set up this functionality,
				// a flag is set indicating that the authenticated users role was selected.
				$au = TRUE;
			}
			// Selected roles other than authenticated user
			elseif($rid)
			{
				// Get the RID to be used for logging later
				$selected_roles[] = $rid;
				// Only execute the following code if the authenticated users role was not selected
				if(!$au)
				{
					// Get a list of UIDs for the users in that role
					$uids = db_query
					(
						'SELECT entity_id ' .
						'FROM {user__roles} ' .
						'WHERE roles_target_id = :rid',
						array(':rid' => $rid)
					)->fetchCol();

					// If the role has any users, force them to change their password
					if(count($uids))
					{
						$this->passwordChangeService->forceUsersPasswordChange($uids);
					}
				}
			}
		}

		// If any roles have had a forced password change, enter the following conditional
		if(count($selected_roles))
		{
			// Log the time of the force for the role
			$query = $this->connection->update('force_password_change_roles')
				->fields(array('last_force' => REQUEST_TIME))
				->condition('rid', $selected_roles, 'IN')
				->execute();

			// Build an list of the names of the roles that had their password change forced
			$roles = user_role_names(TRUE);
			$items = array();
			foreach($selected_roles as $sr)
			{
				$items[] = $roles[$sr];
			}

			$list = array
			(
				'#theme' => 'item_list',
				'#items' => $items,
			);

			$item_list = new FormattableMarkup('@item_list', array('@item_list' => render($list)));
			// Set a message informing the user of the roles that had a forced password change
			if($form_state->getValue('login_only'))
			{
				drupal_set_message(t('Users in the following roles will be required to change their password on their next login: @roles', array('@roles' => $item_list)), 'status');
			}
			else
			{
				drupal_set_message(t('Users in the following roles will be required to immediately change their password: @roles', array('@roles' => $item_list)), 'status');
			}
		}

		$this->configFactory->getEditable('force_password_change.expire_password')
			->set('expire', (bool) $form_state->getValue('expire_password'))
			->save();

		// Prepare the insert query for new roles that have not had their password expiry set
		$query = $this->connection->insert('force_password_change_expiry')
			->fields(array('rid', 'expiry', 'weight'));
		// Set a flag as the insert query will only need to be executed if new roles exist
		$execute_query = FALSE;
		// Loop through the roles and either update their row in the expiry table, or add to the
		// insert query
		$time_periods = array
		(
			'hour' => 60 * 60,
			'day' => 60 * 60 * 24,
			'week' => 60 * 60 * 24 * 7,
			'year' => 60 * 60 * 24 * 365,
		);

		foreach($form_state->getValue('table') as $rid => $expiry)
		{
			// Convert the selected time period into a UNIX timestamp
			// that will be used to calculate whether or not the password has expired.
			$time_period = $expiry['time']['time_quantity'] * $time_periods[$expiry['time']['time_period']];
			// If the role already exists in the database, and the value has changed,
			if($form_state->getValue(['expiry_data', $rid])
			  && ($time_period != $form_state->getValue(['expiry_data', $rid, 'expiry'])
			  || $expiry['weight'] != $form_state->getValue(['expiry_data', $rid, 'weight'])))
			{
				$update_query = $this->connection->update('force_password_change_expiry')
					->fields(array('expiry' => $time_period, 'weight' => $expiry['weight']))
					->condition('rid', $rid)
					->execute();
			}
			// If the role doesn't have a row in the database, add to the insert query
			elseif(!$form_state->getValue(['expiry_data', $rid]))
			{
				$query->values(array('rid' => $rid, 'expiry' => $time_period, 'weight' => $expiry['weight']));
				// Set the flag indicating that the query should be executed
				$execute_query = TRUE;
			}
		}

		// Execute the query only if new roles were found
		if($execute_query)
		{
			$query->execute();
		}
	}
}
