<?php

/**
 * @file Contains Drupal\force_password_change\Form\AdminForm
 *
 * Form handler for the form to add users as a friend
 */

namespace Drupal\force_password_change\Form;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\force_password_change\Service\ForcePasswordChangeService;
use Drupal\user\Entity\Role;
use Drupal\user\RoleInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class RoleForceForm extends FormBase
{
	/**
	 * The config factory object
	 *
	 * @var Drupal\Core\Config\ConfigFactory
	 */
	protected $configFactory;

	/**
	 * The database connect object
	 *
	 * @var Drupal\Core\Database\Connection
	 */
	protected $connection;

	/**
	 * The force password change service
	 *
	 * @var Drupal\force_password_change\Service\ForcePasswordChangeService
	 */
	protected $passwordChangeService;

	/**
	 * The date formatter service
	 *
	 * @var Drupal\Core\Datetime\DateFormatter
	 */
	protected $dateFormatter;

	/**
	 * Constructs an AdminForm object.
	 *
	 * @param \Drupal\Core\Config\ConfigFactory $configFactory
	 *   The config factory service.
	 * @param \Drupal\Core\Database\Connection $connection
	 *   The database connection object.
	 * @param Drupal\force_password_change\Service\ForcePasswordChangeService $passwordChangeService
	 *   The force password change service
	 * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
	 *   The date formatter service
	 */
	public function __construct(ConfigFactory $configFactory, Connection $connection, ForcePasswordChangeService $passwordChangeService, DateFormatter $dateFormatter)
	{
		$this->configFactory = $configFactory;
		$this->connection = $connection;
		$this->passwordChangeService = $passwordChangeService;
		$this->dateFormatter = $dateFormatter;
	}

	/**
	 * {@inheritdoc}
	 */
	public static function create(ContainerInterface $container)
	{
		return new static (
			$container->get('config.factory'),
			$container->get('database'),
			$container->get('force_password_change.service'),
			$container->get('date.formatter')
		);
	}

	public function getFormId()
	{
		return 'force_password_change_role_admin_form';
	}

	public function buildForm(array $form, FormStateInterface $form_state, $role = NULL)
	{
		if($role)
		{
			if(\Drupal::config('force_password_change.when_to_check')->get('login_only'))
			{
				$description = t('Users will be required to change their password upon their next login.');
			}
			else
			{
				$description = t('Users who are not signed in will be required to change their password immediately upon login. Users who are currently signed in will be required to change their password upon their next page click, but after changing their password will be redirected back to the page they were attempting to access.');
			}
			$description .= '<br />' . t('Note: When you return to this page, this box will be unchecked. This is because this setting is a trigger, not a persistant state.');

			$form['force_password_change'] = array
			(
				'#type' => 'checkbox',
				'#title' => t('Force users in this role to change their password'),
				'#description' => $description,
				'#weight' => -1,
			);

			$form['role'] = array
			(
				'#type' => 'value',
				'#value' => $role,
			);

			$form['submit'] = array
			(
				'#type' => 'submit',
				'#value' => t('Force Password Change'),
			);
		}
		else
		{
			$form['no_role'] = array
			(
				'#prefix' => '<p>',
				'#suffix' => '</p>',
				'#markup' => t('No role supplied'),
			);
		}

		return $form;
	}

	public function submitForm(array &$form, FormStateInterface $form_state)
	{
		// Only execute the code if the checkbox was selected
		if($form_state->getValue('force_password_change'))
		{
			$role = $form_state->getValue('role');

			// If the role is the authenticated users role, force the change for
			// for all users
			if($role->id() == 'authenticated')
			{
				$this->passwordChangeService->forceUsersPasswordChange();
			}
			// Execute the following code if the role is anything other than
			// the authenticated users role
			else
			{
				// Get all UIDS for all members of the role
				$uids = $this->connection->query
				(
					'SELECT entity_id ' .
					'FROM {user__roles} ' .
					'WHERE roles_target_id = :rid',
					array(':rid' => $role->id())
				)->fetchCol();

				// If any users are found, force their password change
				if(count($uids))
				{
					$this->passwordChangeService->forceUsersPasswordChange($uids);
				}
			}

			// Log the force time for the role for statistics sake
			$query = $this->connection->update('force_password_change_roles')
				->fields(array('last_force' => REQUEST_TIME))
				->condition('rid', $role->id())
				->execute();

			// Set a message depending on the site settings
			if(\Drupal::config('force_password_change.when_to_check')->get('login_only'))
			{
				drupal_set_message(t('Users in this role will be required to change their password on next login'));
			}
			else
			{
				drupal_set_message(t('Users in this role will be required to immediately change their password'));
			}
		}
	}
}
