<?php

/**
 * @file Contains Drupal\force_password_change\Service\ForcePasswordChangeService
 *
 * Maps database requests for data regarding friendships
 */

namespace Drupal\force_password_change\Service;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Database\Connection;
use Drupal\Core\Session\AccountProxy;
use Drupal\user\UserData;

class ForcePasswordChangeService implements ForcePasswordChangeServiceInterface
{
	/**
	 * The database connection
	 *
	 * @var \Drupal\Core\Database\Connection
	 */
	protected $connection;

	/**
	 * The current user
	 *
	 * @var \Drupal\Core\Session\AccountProxy
	 */
	protected $currentUser;

	/**
	 * The config factory object
	 *
	 * @var \Drupal\Core\Config\ConfigFactory
	 */
	protected $configFactory;

	/**
	 * The config factory object
	 *
	 * @var \Drupal\user\UserData
	 */
	protected $userData;

	/**
	 * Constructs a ForcePasswordChangeService object.
	 *
	 * @param \Drupal\Core\Database\Connection $connection
	 *   The database connection
	 * @param \Drupal\Core\Session\AccountProxy $currentUser
	 *   The current user
	 * @param \Drupal\Core\Config\ConfigFactory $configFactory
	 *   The config factory
	 * @param \Drupal\user\UserData $userData
	 *   The user data service
	 */
	public function __construct(Connection $connection, AccountProxy $currentUser, ConfigFactory $configFactory, UserData $userData)
	{
	  $this->connection = $connection;
	  $this->currentUser = $currentUser;
	  $this->configFactory = $configFactory;
	  $this->userData = $userData;
	}

	public function forceUsersPasswordChange($uids = array())
	{
		$query = $this->connection->update('users_data')
			->fields(array('value' => 1))
			->condition('module', 'force_password_change')
			->condition('name', 'force_password_change');

		if(!empty($uids))
		{
			$query->condition('uid', $uids, 'IN');
		}

		$query->execute();

		$query = $this->connection->update('force_password_change_users')
			->fields(array('last_force' => REQUEST_TIME));

		if(!empty($uids))
		{
			$query->condition('uid', $uids, 'IN');
		}

		$query->execute();
	}

	public function checkForForce()
	{
		// Default is to not redirect
		$redirect = FALSE;

		// If the user's account has been flagged for expiry, a redirect is required
		if($this->userData->get('force_password_change', $this->currentUser->id(), 'force_password_change'))
		{
			$redirect = 'admin_forced';
		}
		// Only check to see if their password has expired if password expiry
		// is turned on in the module settings page.
		elseif($this->configFactory->get('force_password_change.expire_password')->get('expire'))
		{
			// The user's account has not been flagged for password expiry. Check to see
			// if their password has expired according to the rules of the module.

			// First thing is to check the time of their last password change,
			// and the time of their account creation
			$query = $this->connection->select('force_password_change_users', 'fpcu');
			$alias = $query->join('users_field_data', 'ufd', 'ufd.uid = fpcu.uid');
			$query
				->fields('fpcu', array('last_password_change'))
				->fields($alias, array('created'))
				->condition($alias . '.uid', $this->currentUser->id());
			$user_data = $query->execute()->fetchObject();

			// Get the time period after which their password should expire
			// according to the rules laid out in the module settings page. Only the
			// role with the highest priority is retrieved
			$query = $this->connection->select('force_password_change_expiry', 'fpce');
			$expiry = $query
				->fields('fpce', array('expiry'))
				->condition('fpce.rid', array_keys($this->currentUser->getRoles()), 'IN')
				->orderBy('fpce.weight')
				->range(0, 1)
				->addTag('force_password_change_expiry_check')
				->execute()
				->fetchField();

			// Test to see if their password has expired
			if($expiry && ($user_data->last_password_change != '' && (REQUEST_TIME - $expiry) > $user_data->last_password_change) || ($user_data->last_password_change == '' && (REQUEST_TIME - $expiry) > $user_data->created))
			{
				// Their password has expired, so their user account is flagged
				// and the expiration time period is returned, which will trigger the redirect
				// and be used to generate the message shown to the user
				$query = $this->connection->update('users')
					->fields(array('force_password_change' => 1))
					->condition('uid', $this->currentUser->id())
					->execute();

				$redirect = 'expired';
			}
		}

		return $redirect;
	}

	/**
	 * Converts a time period of password expiry dates to the textual representation
	 * of the number of years, months, days or hours that it represents
	 */
	function getTextDate($timestamp)
	{
		$year = 60 * 60 * 24 * 365;
		if($timestamp % $year === 0)
		{
			$time_period = $timestamp / $year;
			$time_period = ($time_period > 1) ? $time_period . ' ' . t('years') : t('year');
		}
		else
		{
			$week = 60 * 60 * 24 * 7;
			if($timestamp % $week === 0)
			{
				$time_period = $timestamp / $week;
				$time_period = ($time_period > 1) ? $time_period . ' ' . t('weeks') : t('week');
			}
			else
			{
				$day = 60 * 60 * 24;
				if($timestamp % $day === 0)
				{
					$time_period = $timestamp / $day;
					$time_period = ($time_period > 1) ? $time_period . ' ' . t('days') : t('day');
				}
				else
				{
					$hour = 60 * 60;
					if($timestamp % $hour === 0)
					{
						$time_period = $timestamp / $hour;
						$time_period = ($time_period > 1) ? $time_period . ' ' . t('hours') : t('hour');
					}
				}
			}
		}

		return $time_period;
	}
}
